package com.toone.itop.formula.rule.inte;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.toone.exception.BusinessException;
import com.toone.itop.formula.spi.Func;
import com.toone.itop.formula.spi.Function;
import com.toone.itop.metadata.apiserver.DataView;
import com.toone.itop.rule.apiserver.model.RuleContext;
import com.toone.itop.workflow.engine.apiserver.model.TProcessInstance;
import com.toone.itop.workflow.engine.apiserver.model.Task;
import com.toone.itop.workflow.engine.vbase.common.apiserver.factory.VBASEWorkFlowServiceFactory;
import com.toone.itop.workflow.engine.vbase.common.apiserver.task.TaskQuery;
import com.toone.itop.workflow.engine.vbase.common.apiserver.util.WorkFlowOrganizationUtils;
import com.toone.itop.workflow.engine.vbase.common.spiserver.rule.WorkFlowExecutionUnitFuncRule;
import com.toone.itop.workflow.pvm.apiserver.Activity;
import com.toone.itop.workflow.pvm.apiserver.ProcessExecution;
import com.toone.itop.workflow.pvm.apiserver.state.State;
import com.toone.util.ArrayUtils;
import com.toone.util.CollectionUtils;
import com.toone.util.StringUtils;
import com.toone.vcore.component.annotations.Component;
import com.toone.vcore.component.annotations.Instantiate;
import com.toone.vcore.component.annotations.Provides;

@Component
@Provides(specifications = Function.class)
@Instantiate
@Func(name = "Serverrule_test_GetExecutionSuperior")
public class Serverrule_test_GetExecutionSuperior extends
		WorkFlowExecutionUnitFuncRule {
	/**
	 * 规则定义参数Key
	 */
	private static enum RCKey {
		// 1取上一步执行人的上级2指定流程变量
		sourceType,
		// 流程变量编码
		varCode,
		// 是否获取直接上级
		isGetSuperior,
		// 是否获取直属上级
		isGetDirectlySuperior
	}

	@Override
	public Object enhancedEvaluate(RuleContext ruleContext) {
		ruleContext.setResultToJson(false);

		Map<String, Object> ruleConfigInParams = ruleContext.getRuleConfig()
				.getConfigParams();

		String sourceTypeVar = (String) ruleConfigInParams.get(RCKey.sourceType
				.name());
		String varCodeVar = (String) ruleConfigInParams.get(RCKey.varCode
				.name());
		boolean isGetSuperior = Boolean
				.parseBoolean((String) ruleConfigInParams
						.get(RCKey.isGetSuperior.name()));
		boolean isGetDirectlySuperior = Boolean
				.parseBoolean((String) ruleConfigInParams
						.get(RCKey.isGetDirectlySuperior.name()));
		if ("1".equals(sourceTypeVar)) {
			// 取上一步执行人
			return getSuperior(ruleContext, isGetSuperior,
					isGetDirectlySuperior);
		} else {
			// 指定执行人
			return getWorkFlowVarSuperior(ruleContext, varCodeVar,
					isGetSuperior, isGetDirectlySuperior);
		}
	}

	// 根据流程变量获取上级
	private DataView getWorkFlowVarSuperior(RuleContext ruleContext,
			String varCode, boolean isGetSuperior, boolean isGetDirectlySuperior) {
		Map<String, Object> flowVariables = super.getFlowVariables(ruleContext);
		// 创建dataview对象数据
		DataView dataview = WorkFlowOrganizationUtils
				.createExecutionUnitDataView();

		// 获取流程变量中对应外部制定执行人ID串对应的流程变量值
		String varValue = (String) flowVariables.get(varCode);
		if (StringUtils.isEmpty(varValue)) {
			throw new BusinessException("【执行人函数：人资系统-获取指定人员的上级】运行异常:"
					+ "根据流程变量key = " + varCode + "获取变量值" + varValue + "为空");
		}
		varValue = varValue.trim();
		// 多个执行人id用逗号分隔
		String[] userIds = varValue.split(",");
		if (userIds != null && userIds.length > 0) {
			String[] superiorsIds = null;
			String[] directSuperiorsIds = null;
			if (isGetSuperior) {
				// 根据员工id取员工管理找直接上级
				superiorsIds = getSuperiorsByUserID(Arrays.asList(userIds));
			}
			if (isGetDirectlySuperior) {
				// 根据员工id取员工管理找直属上级
				directSuperiorsIds = getDirectSuperiorsByUserID(Arrays
						.asList(userIds));
			}
			String[] totalIds = conCatStrs(superiorsIds, directSuperiorsIds);
			// 组建机构用户列表
			if (totalIds != null && totalIds.length > 0) {
				WorkFlowOrganizationUtils
						.getOrgUserListByUserIdsWithSingleQuery(dataview,
								totalIds);
			}
		}

		return dataview;
	}

	// 取上一步执行人
	private DataView getSuperior(RuleContext ruleContext,
			boolean isGetSuperior, boolean isGetDirectlySuperior) {
		List<String> vSysUserIds = new ArrayList<String>();
		// 创建dataview对象数据
		DataView dataview = WorkFlowOrganizationUtils
				.createExecutionUnitDataView();
		// 当前流程实例
		TProcessInstance processInstance = super
				.getCurrentProcessInstance(ruleContext);
		// 如果没有当前流程实例则认为是流程启动的时候请求执行人函数的，此时上一步执行人为当前登录人
		if (processInstance == null) {
			String loginUserId = null;
			Map<String, Object> flowVariables = super
					.getFlowVariables(ruleContext);
			if (null != flowVariables
					&& flowVariables.containsKey("StartExecutionUnitId")) {
				loginUserId = (String) flowVariables
						.get("StartExecutionUnitId");
			}
			if (StringUtils.isEmpty(loginUserId)) {
				throw new BusinessException(
						"【执行人函数Hr_GetExecutionSuperior：取上一步执行人的直接上级】运行异常："
								+ "通过系统变量:v_sys_userID,获取当前登录人id为空");
			}
			vSysUserIds.add(loginUserId);
		} else {
			Activity currentActivity = super.getCurrentActivity(ruleContext);
			String curActivityId = currentActivity.getId();

			boolean isRejectOrSkip = isRejectOrSkip(ruleContext);

			ProcessExecution fromProcessExecution = getFromProcessExecution(ruleContext);
			if (isRejectOrSkip) {
				// 来源当前信号为回退或跳转，需要找到回退或跳转活动的最后一次运行过的活动实例的来源活动。
				fromProcessExecution = getMatchExecutionFromAllRunnedExecutions(
						ruleContext, curActivityId);
				fromProcessExecution = fromProcessExecution.getFrom();
			}

			if (null != fromProcessExecution) {

				// 根据指定的活动实例ID寻找已完成的任务列表
				TaskQuery taskQuery = VBASEWorkFlowServiceFactory
						.getVBASEWorkFlowService().getTaskQuery();
				List<Task> matchTaskLst = taskQuery
						.findByExecutionIdAndStatesOrderByOrderNoASC(
								fromProcessExecution.getId(),
								new String[] { State.Running.name(),
										State.Complete.name() });
				if (!CollectionUtils.isEmpty(matchTaskLst)) {
					// 收集匹配的活动上的已完成的任务对应的任务执行人
					for (Task matchTask : matchTaskLst) {
						String ownerId = matchTask.getOwner();
						vSysUserIds.add(ownerId);
					}
				}
			}
		}
		if (vSysUserIds != null && vSysUserIds.size() > 0) {
			String[] superiorsIds = null;
			String[] directSuperiorsIds = null;
			if (isGetSuperior) {
				// 根据员工id取员工管理找直接上级
				superiorsIds = getSuperiorsByUserID(vSysUserIds);
			}
			if (isGetDirectlySuperior) {
				// 根据员工id取员工管理找直属上级
				directSuperiorsIds = getDirectSuperiorsByUserID(vSysUserIds);
			}
			String[] totalIds = conCatStrs(superiorsIds, directSuperiorsIds);
			// 组建机构用户列表
			if (totalIds != null && totalIds.length > 0) {
				WorkFlowOrganizationUtils
						.getOrgUserListByUserIdsWithSingleQuery(dataview,
								totalIds);
			}
		}

		return dataview;
	}

	// 合并两个字符串数组
	private String[] conCatStrs(String[] superiorsIds,
			String[] directSuperiorsIds) {
		if (superiorsIds != null && directSuperiorsIds != null) {
			return (String[]) ArrayUtils.addArrayElement(superiorsIds,
					directSuperiorsIds);
		}
		if (superiorsIds == null && directSuperiorsIds == null) {
			return null;
		}
		if (superiorsIds != null) {
			return superiorsIds;
		}
		if (directSuperiorsIds != null) {
			return directSuperiorsIds;
		}
		return null;
	}

	// 获取直接上级
	private String[] getSuperiorsByUserID(List<String> vSysUserIds) {
		List<String> results = new ArrayList<String>();
		// 根据执行人Id数组vSysUserIds按照自己的业务规则查找直接上级Id列表并返回
		// ....此处为自己的业务规则
		return results.toArray(new String[results.size()]);
	}

	// 获取直属上级
	private String[] getDirectSuperiorsByUserID(List<String> vSysUserIds) {
		List<String> results = new ArrayList<String>();
		// 根据执行人Id数组vSysUserIds按照自己的业务规则查找直属上级Id列表并返回
		// ....此处为自己的业务规则
		return results.toArray(new String[results.size()]);
	}

}